@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: parse.c,v 1.84 2001/02/03 18:46:50 nishida Exp $
+ * $Id: parse.c,v 1.86 2001/09/17 14:48:30 nishida Exp $
  */
 
 #include "mgp.h"
@@ -754,6 +754,8 @@ secondpass()
 		for (cp = tab_control[l]; cp; cp = cp->ct_next) {
 			if (cp->ct_op == CTL_PREFIX)
 				cp->ct_op = CTL_TABPREFIX;
+			if (cp->ct_op == CTL_PREFIXN)
+				cp->ct_op = CTL_TABPREFIXN;
 		}
 	}
 }
@@ -784,6 +786,7 @@ thirdpass()
 	struct ctrl *tmpint[10];
 	struct ctrl *tmplong[10];
 	struct ctrl *tmpvoid[10];
+	struct ctrl *tmparea[10];
 	struct ctrl sentinel;
 	int i;
 
@@ -792,6 +795,7 @@ thirdpass()
 	memset(tmpint, 0, sizeof(tmpint));
 	memset(tmplong, 0, sizeof(tmplong));
 	memset(tmpvoid, 0, sizeof(tmpvoid));
+	memset(tmparea, 0, sizeof(tmparea));
 
 	/* default value for page 1 */
 	tmpstr[0] = ctlalloc1(CTL_PREFIX);
@@ -839,6 +843,11 @@ thirdpass()
 	tmpint[3] = ctlalloc1(CTL_QUALITY);
 	tmpint[3]->cti_value = DEFAULT_BQUALITY;
 	tmpvoid[0] = ctlalloc1(CTL_LEFT);
+	tmparea[0] = ctlalloc1(CTL_AREA);
+	tmparea[0]->ctar_width = 100;
+	tmparea[0]->ctar_height = 100;
+	tmparea[0]->ctar_xoff = 0;
+	tmparea[0]->ctar_yoff = 0;
 
 	/* for page 1 */
 	cp = &sentinel;
@@ -886,6 +895,16 @@ thirdpass()
 		cp->ct_next = ctlalloc1(tmpvoid[i]->ct_op);
 		cp = cp->ct_next;
 	}
+	for (i = 0; i < 10; i++) {
+		if (!tmparea[i])
+			continue;
+		cp->ct_next = ctlalloc1(tmparea[i]->ct_op);
+		cp->ct_next->ctar_width = tmparea[i]->ctar_width;
+		cp->ct_next->ctar_height = tmparea[i]->ctar_height;
+		cp->ct_next->ctar_xoff = tmparea[i]->ctar_xoff;
+		cp->ct_next->ctar_yoff = tmparea[i]->ctar_yoff;
+		cp = cp->ct_next;
+	}
 	carryover[0] = sentinel.ct_next;
 
 	/*
@@ -943,6 +962,8 @@ thirdpass()
 				case CTL_RIGHT: tmpvoid[0] = cp; break;
 				case CTL_CENTER: tmpvoid[0] = cp; break;
 				case CTL_LEFTFILL: tmpvoid[0] = cp; break;
+
+				case CTL_AREA: tmpvoid[0] = cp; break;
 				}
 			}
 		}
@@ -992,6 +1013,16 @@ thirdpass()
 			cp->ct_next = ctlalloc1(tmpvoid[i]->ct_op);
 			cp = cp->ct_next;
 		}
+		for (i = 0; i < 10; i++) {
+			if (!tmparea[i])
+				continue;
+			cp->ct_next = ctlalloc1(tmparea[i]->ct_op);
+			cp->ct_next->ctar_width = tmparea[i]->ctar_width;
+			cp->ct_next->ctar_height = tmparea[i]->ctar_height;
+			cp->ct_next->ctar_xoff = tmparea[i]->ctar_xoff;
+			cp->ct_next->ctar_yoff = tmparea[i]->ctar_yoff;
+			cp = cp->ct_next;
+		}
 
 		carryover[page] = sentinel.ct_next;
 	}
