@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: print.c,v 1.94 2001/03/05 13:16:14 nishida Exp $
+ * $Id: print.c,v 1.97 2001/09/17 14:48:30 nishida Exp $
  */
 /*
  * Paper size selection code is based on psutil.c by Angus J. C. Duggan
@@ -53,6 +53,8 @@ static int colorps = 0;
 static int painticon = 0;
 static u_int curlinenum;
 static u_int curpagenum;
+static u_int xprefix = 0;
+static u_int tabxprefix = 0;
 static u_int valign = VL_BOTTOM;
 
 #ifdef FREETYPE
@@ -234,7 +236,6 @@ static void scan_font __P((u_int));
 static void print_page __P((u_int));
 static void print_init __P((void));
 static void print_pageinit __P((u_int));
-static void print_pagebackground __P((void));
 static void process_direc_print __P((struct ctrl *));
 static void line_start __P((void));
 static void line_end __P((void));
@@ -430,6 +431,7 @@ print_page(page)
 			process_direc_print(cp);
 	}
 
+	fprintf(fp, "grestore\n\n");
 	fprintf(fp, "showpage\n\n");
 }
 
@@ -534,8 +536,13 @@ print_init()
 	for (p = aligns; *p; p++) {
 		fprintf(fp, "/%snewlinecheck {\n", *p);
 		fprintf(fp, "  currentpoint pop add XMARGIN XBODY add gt {\n");
+#if 1
+		fprintf(fp, "    NL %sdefxpos xpos ypos charsize 2 div sub moveto\n",
+			*p);
+#else
 		fprintf(fp, "    NL %sdefxpos xpos ypos charsize sub moveto\n",
 			*p);
+#endif
 		fprintf(fp, "  } if\n");
 		fprintf(fp, "} bind def\n");
 	}
@@ -578,21 +585,25 @@ static void
 print_pageinit(page)
 	u_int page;
 {
+
+	window_width  = paper->width - paper_xmargin * 2;
+	window_height = paper->height - paper_ymargin * 2;
 	fprintf(fp, "%%%%Page: %d %d\n", page, page);
+#if 1 /*XXX*/
+	fprintf(fp, "/ypos YMARGIN -1 mul 4 sub def\n");
+	fprintf(fp, "/xpos 0 def\n");
+#else
 	fprintf(fp, "/ypos YMARGIN -1 mul def\n");
 	fprintf(fp, "/xpos XMARGIN def\n");
+#endif
 	fprintf(fp, "initcharsize\n");
-}
-
-static void
-print_pagebackground()
-{
 	if (!colorps)
 		fprintf(fp, "90 rotate newpath writebox\n");
 	else {
 		setpencolor(back);
 		fprintf(fp, "90 rotate newpath writeboxfill\n");
 	}
+	fprintf(fp, "gsave\n");
 }
 
 static void
@@ -711,10 +722,18 @@ process_direc_print(cp)
 		curprefix = cp->ctc_value;
 		break;
 
+	case CTL_PREFIXN:
+		xprefix = window_width * cp->ctf_value / 100;
+		break;
+
 	case CTL_TABPREFIX:
 		tabprefix = cp->ctc_value;
 		break;
 
+	case CTL_TABPREFIXN:
+		tabxprefix = window_width * cp->ctf_value / 100;
+		break;
+
 	case CTL_PREFIXPOS:
 		if (tabprefix)
 			print_line(tabprefix, 1);
@@ -769,6 +788,20 @@ process_direc_print(cp)
 		valign = cp->cti_value;
 		break;
 
+	case CTL_AREA:
+		window_width = (paper->width - paper_xmargin * 2) * cp->ctar_width / 100;
+		window_height = (paper->height - paper_ymargin * 2) * cp->ctar_height / 100;
+		fprintf(fp, "grestore\ngsave\n");
+		fprintf(fp, "%f dup scale\n", cp->ctar_height / 100.0);
+		fprintf(fp, "WIDTH XMARGIN 2 mul sub %f mul 0 translate\n", 
+		    (double)cp->ctar_xoff / cp->ctar_height);
+		fprintf(fp, "/XBODY WIDTH XMARGIN 2 mul sub %f mul def\n",
+		    (double)cp->ctar_width / cp->ctar_height);
+		fprintf(fp, "/ypos YMARGIN -1 mul 4 sub HEIGHT %f mul sub def\n",
+		    (double)cp->ctar_yoff / cp->ctar_height);
+		fprintf(fp, "/xpos 0 def\n");
+		break;
+
 	default:
 		break;
 	}
@@ -780,13 +813,6 @@ line_start()
 	linewidth = 0;
 	nimagepool = 0;
 	ntextpool = 0;
-
-	if (curlinenum == 0){
-		print_pagebackground();
-#if 0
-		fprintf(fp, "initcharsize %d setcharsize\n", char_size[0]);
-#endif
-	}
 }
 
 static void
@@ -873,6 +899,7 @@ line_end()
 	 *	}
 	 */
 	fprintf(fp, "/totlen exch def\n");
+	fprintf(fp, "/totlen totlen %d add def\n", tabxprefix ? tabxprefix:xprefix);
 	fprintf(fp, "/inmargin true def /fillzero XMARGIN def\n");
 	fprintf(fp, "%sdefxpos ", alignstr);
 
@@ -900,6 +927,7 @@ line_end()
 		fprintf(fp, " { 0 } { imgsize charsize sub 2 div } ifelse sub def\n");
 		break;
 	}
+	fprintf(fp, "/xpos xpos %d add def\n", tabxprefix ? tabxprefix:xprefix);
 	fprintf(fp, "xpos yypos ymax sub moveto\n");
 
 	for (i = 0; i < ntextpool; i++) {
@@ -979,6 +1007,7 @@ line_end()
 done:
 	fprintf(fp, "NL\n");
 	tabprefix = NULL;
+	tabxprefix = 0;
 }
 
 static void
